-- SpeedWarp
declare('SpeedWarp', SpeedWarp or {})
SpeedWarp.version = 1.3
SpeedWarp.timer = Timer()

SpeedWarp.defaults = {}
SpeedWarp.defaults.delay = 2
SpeedWarp.defaults.factor = 3
SpeedWarp.defaults.max = 179

SpeedWarp.active = ( gkini.ReadInt('SpeedWarp', 'active', 1) ~= 0 )
SpeedWarp.short_delay = gkini.ReadInt('SpeedWarp', 'delay', SpeedWarp.defaults.delay)
SpeedWarp.long_delay = 100
SpeedWarp.delay = SpeedWarp.long_delay
SpeedWarp.factor = gkini.ReadInt('SpeedWarp', 'factor', SpeedWarp.defaults.factor)
SpeedWarp.max = gkini.ReadInt('SpeedWarp', 'max', SpeedWarp.defaults.max)
SpeedWarp.orig = Game.GetCVar('fov')
SpeedWarp.fov_is_fine = false

--resets settings to defaults
function SpeedWarp:set_defaults()
	self.short_delay = self.defaults.delay
	self.factor = self.defaults.factor
	self.max = self.defaults.max
	gkini.WriteInt('SpeedWarp', 'delay', self.short_delay)
	gkini.WriteInt('SpeedWarp', 'factor', self.factor)
	gkini.WriteInt('SpeedWarp', 'max', self.max)
	print("\127FFFFFFSpeedWarp reset to defaults\127o")
end

--performs cubic interpolation between 's' and 'e', where 'x' is the
--percentage of the distance traveled.  'x' is clamped to [0, 1]
function SpeedWarp.cubic_interp(s, e, x)
	x = math.min(x, 1)
	x = math.max(x, 0)
	return( 2*(s-e)*x^3 - 3*(s-e)*x^2 + s)
end

--this is the loop
function SpeedWarp:run()
	if self.active then
		self:action()
		self.timer:SetTimeout(self.delay, function() self:run() end)
	end
end

--this is where the actual logic happens
function SpeedWarp:action()
	local speed = GetActiveShipSpeed()
	local max_speed = GetActiveShipMaxSpeed()
	local fov = self.orig
	
	--if no ship, do nothing
	if speed == nil or max_speed == nil then return end
	
	--only run fast when necessary
	if speed > max_speed - 5 then
		self.delay = self.short_delay
	else
		self.delay = self.long_delay
	end
	
	if speed > max_speed then
		--compute and set the new fov
		local x = (speed - max_speed)/(self.factor*max_speed)
		fov = self.cubic_interp(self.orig, self.max, x)
		gkinterface.SetFOV(fov)
		self.fov_is_fine = false
	else
		--don't want to constantly set the fov if we don't have to, because then we can't use the zoom feature
		if not self.fov_is_fine then
			gkinterface.SetFOV(self.orig)
			self.fov_is_fine = true
		end
	end
end

function SpeedWarp:enable(nosave)
	self.active = true
	self:run()
	if not nosave then
		gkini.WriteInt('SpeedWarp', 'active', 1)
		print("\127FFFFFFSpeedWarp enabled\127o")
	end
end

function SpeedWarp:disable(nosave)
	self.active = false
	gkinterface.SetFOV(self.orig)
	if not nosave then
		gkini.WriteInt('SpeedWarp', 'active', 0)
		print("\127FFFFFFSpeedWarp disabled\127o")
	end
end

--process the commands
function SpeedWarp.proc(_,data)
	if (data == nil) then
		if SpeedWarp.active then
			SpeedWarp:disable()
		else
			SpeedWarp:enable()
		end
	elseif (data[1] == "enable") then
		SpeedWarp:enable()
	elseif (data[1] == "disable") then
		SpeedWarp:disable()
	elseif (data[1] == "delay" and data[2] ~= nil) then
		SpeedWarp.short_delay = tonumber(data[2])
		gkini.WriteInt('SpeedWarp', 'delay', SpeedWarp.short_delay)
	elseif (data[1] == "max" and data[2] ~= nil) then
		SpeedWarp.max = tonumber(data[2])
		gkini.WriteInt('SpeedWarp', 'max', SpeedWarp.max)
	elseif (data[1] == "factor" and data[2] ~= nil) then
		SpeedWarp.factor = tonumber(data[2])
		gkini.WriteInt('SpeedWarp', 'factor', SpeedWarp.factor)
	elseif (data[1] == "defaults") then
		SpeedWarp:set_defaults()
	elseif (data[1] == "settings") then
		print("\127FFFFAASpeedWarp \127o\127FFFFFFcurrent settings:\127o")
		if SpeedWarp.active then
			print("\127999999* active: \127o\127FFFFFFtrue\127o")
		else
			print("\127999999* active: \127o\127FFFFFFfalse\127o")
		end
		print("\127999999* delay: \127o\127FFFFFF"..SpeedWarp.short_delay.."\127o")
		print("\127999999* max: \127o\127FFFFFF"..SpeedWarp.max.."\127o")
		print("\127999999* factor: \127o\127FFFFFF"..SpeedWarp.factor.."\127o")
	else
		print("\127FF6600SpeedWarp \127o\127FFFFAAversion "..SpeedWarp.version.."\127o")
		print("\127FFFFFF/speedwarp    \127o\127999999 toggle\127o")
		print("\127FFFFFF/speedwarp enable|disable   \127o\127999999 enable/disable\127o")
		print("\127FFFFFF/speedwarp settings    \127o\127999999 show current settings\127o")
		print("\127FFFFFF/speedwarp delay <n>    \127o\127999999 set delay to n milliseconds\127o")
		print("\127FFFFFF/speedwarp max <n>    \127o\127999999 set max fov to n degrees\127o")
		print("\127FFFFFF/speedwarp factor <n>    \127o\127999999 larger n results in a slower fov increase\127o")
		print("\127FFFFFF/speedwarp defaults    \127o\127999999 reset settings to defaults\127o")
	end
end

RegisterUserCommand("speedwarp",SpeedWarp.proc)

--if SpeedWarp is enabled, start it when the player enters game
if SpeedWarp.active then
	RegisterEvent(function() SpeedWarp:enable(true) end, "PLAYER_ENTERED_GAME")
end

--make sure SpeedWarp is turned off before the player logs out
RegisterEvent(function() SpeedWarp:disable(true) end, "PLAYER_LOGGED_OUT")

